from setuptools import setup

setup(name='sjwt',
      version='0.3.10',
      description='Generate, check token, and get payload from token',
      packages=['sjwt'],
      author_email='sagrityanin4@gmail.com',
      zip_safe=False)
